<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>

    <head>
        <title>{{ title }}</title>
        <link rel="stylesheet" href="help:/kdoctools5-common/kde-default.css" type="text/css" />
        <link rel="stylesheet" href="help:/kdoctools5-common/kde-docs.css" type="text/css" />
        <link rel="stylesheet" href="help:/kdoctools5-common/kde-localised.css" type="text/css" />
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    </head>

    <body>
        <div id="content">
        <div id="header"><div id="header_content"><div id="header_left"><div id="header_right">
        <img src="help:/kdoctools5-common/top-kde.jpg" width="36" height="34" alt="K logo" /> {{ title }}</div></div></div></div>
        <div id="contentBody"><div lang="en" class="book">
        <div class="titlepage">
        <div>
        <div><h1 class="title">{{ name }}</h1></div>
        </div>
        <hr />
        </div>

        <div>
        {{ content }}
        </div>

        </div>

        </div>
        </div>
    </body>

</html>
