<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="khelpcenter" version="18">
<MenuBar>
 <Menu name="edit" noMerge="1">
  <text>&amp;Edit</text>
  <Action name="copy_text" />
  <Separator />
  <Action name="selectAll" />
  <Separator />
  <Action name="find" />
 </Menu>
 <Menu name="view" noMerge="1">
  <text>&amp;View</text>
  <Action name="viewDocumentSource" />
  <Action name="setEncoding" />
 </Menu>
 <Menu name="go">
  <text>&amp;Go</text>
  <Action name="prevPage" />
  <Action name="nextPage" />
  <Separator />
  <Action name="back" />
  <Action name="forward" />
<!--
  <Separator />
  <Action name="lastsearch" />
-->
 </Menu>
 <Action name="bookmarks"/>
 <Menu name="settings">
  <Action name="configure_fonts" append="save_merge"/>
 </Menu>
</MenuBar>
<ToolBar name="mainToolBar" noMerge="1">
 <text>Main Toolbar</text>
 <Action name="go_home" />
 <Action name="back" />
 <Action name="forward" />
 <Separator /> 
 <Action name="copy_text" />
 <Action name="find" /> 
 <Separator />
 <Action name="incFontSizes" />
 <Action name="decFontSizes" />
</ToolBar>
</gui>
